from rosemary.sqlquery import *
from wooly.datatable import *

from util import *
from operator import itemgetter

class QmfAdapter(DataAdapter):
    def __init__(self, app, method):
        super(QmfAdapter, self).__init__()

        self.app = app
        self.method = method
        self.default = list()
        self.columns = list()
        self.max_sortable_records = 1000

    def get_count(self, values):
        obj = values['obj']
        args = values['args']

        action = QmfCall(self.app)
        results = action.execute(obj, self.method, args)

        return results and len(results) or 0

    def get_data(self, values, options):
        obj = values['obj']
        args = values['args']

        action = QmfCall(self.app)
        results = action.execute(obj, self.method, args)

        if results.error:
            results.data = self.default

        if isinstance(results.data, dict):
            rows = self.process_results(results.data)
        elif isinstance(results.data, list):
            rows = self.process_list_results(results.data)
        else:
            rows = []

        rows = self.sort_rows(rows, options)

        return rows[options.offset:options.offset + options.limit]

    def process_results(self, results):
        """ take the dict response from the qmf call and return a list of lists """

        records = list()

        if results:
            for key in results:
                row = self.process_record(key, results[key])
                records.append(row)

        return records

    def process_list_results(self, results):
        # the results for this call is a list
        records = list()

        if results:
            for rec in results:
                row = self.process_record(None, rec)
                records.append(row)

        return records

    def process_record(self, key, record):
        field_data = list()
        for column in self.columns:
            try:
                val = record[column.name]
            except KeyError:
                val = 0
            field_data.append(val)
        return field_data

    def sort_rows(self, rows, options):
        if len(rows) > self.max_sortable_records:
            return rows

        sort_field = options.sort_field
        rev = options.sort_ascending == False

        return sorted(rows, key=itemgetter(sort_field.index), reverse=rev)

class ObjectQmfAdapter(QmfAdapter):
    def __init__(self, app, cls):
        super(ObjectQmfAdapter, self).__init__(app, cls._name)

        self.cls = cls

        self.fields_by_attr = dict()

    def add_like_filter(self, attr):
        #TODO: add sql LIKE filtering to QMF results
        pass

class QmfField(DataAdapterField):
    def __init__(self, adapter, column):
        python_type = column.type.python_type

        super(QmfField, self).__init__(adapter, column.name, python_type)

        self.column = column

        self.adapter.columns.append(column)

class ObjectQmfField(QmfField):
    def __init__(self, adapter, attr):
        assert isinstance(adapter, ObjectQmfAdapter), adapter

        super(ObjectQmfField, self).__init__(adapter, attr.sql_column)

        self.attr = attr

        self.adapter.fields_by_attr[self.attr] = self

    def get_title(self, session):
        return self.attr.title or self.attr.name
